"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getOpenApiVersion_1 = require("./utils/getOpenApiVersion");
const getOpenApiSpec_1 = require("./utils/getOpenApiSpec");
const isString_1 = require("./utils/isString");
const v2_1 = require("./openApi/v2");
const v3_1 = require("./openApi/v3");
const postProcessClient_1 = require("./utils/postProcessClient");
const readHandlebarsTemplates_1 = require("./utils/readHandlebarsTemplates");
const writeClient_1 = require("./utils/writeClient");
var HttpClient;
(function (HttpClient) {
    HttpClient["FETCH"] = "fetch";
    HttpClient["XHR"] = "xhr";
})(HttpClient = exports.HttpClient || (exports.HttpClient = {}));
/**
 * Generate the OpenAPI client. This method will read the OpenAPI specification and based on the
 * given language it will generate the client, including the typed models, validation schemas,
 * service layer, etc.
 * @param input The relative location of the OpenAPI spec.
 * @param output The relative location of the output directory.
 * @param httpClient The selected httpClient (fetch or XHR).
 * @param useOptions Use options or arguments functions.
 * @param useUnionTypes Use inclusive union types.
 * @param write Write the files to disk (true or false).
 */
function generate({ input, output, httpClient = HttpClient.FETCH, useOptions = false, useUnionTypes = false, write = true }) {
    try {
        // Load the specification, read the OpenAPI version and load the
        // handlebar templates for the given language
        const openApi = isString_1.isString(input) ? getOpenApiSpec_1.getOpenApiSpec(input) : input;
        const openApiVersion = getOpenApiVersion_1.getOpenApiVersion(openApi);
        const templates = readHandlebarsTemplates_1.readHandlebarsTemplates();
        switch (openApiVersion) {
            case getOpenApiVersion_1.OpenApiVersion.V2: {
                const client = v2_1.parse(openApi);
                const clientFinal = postProcessClient_1.postProcessClient(client, useUnionTypes);
                if (write) {
                    writeClient_1.writeClient(clientFinal, templates, output, httpClient, useOptions);
                }
                break;
            }
            case getOpenApiVersion_1.OpenApiVersion.V3: {
                const client = v3_1.parse(openApi);
                const clientFinal = postProcessClient_1.postProcessClient(client, useUnionTypes);
                if (write) {
                    writeClient_1.writeClient(clientFinal, templates, output, httpClient, useOptions);
                }
                break;
            }
        }
    }
    catch (e) {
        console.error(e);
        process.exit(1);
    }
}
exports.generate = generate;
